import os
import os.path
import sys
import math

from django.db import models
from galeria.settings import PREFIX, STATIC_PATH_PREFIX, UPLOAD_TMP_DIR, \
    LANGUAGE_CODE

from babel.dates import format_datetime
from datetime import datetime, timezone, timedelta

def float2degrees(d):
    i = math.floor(d)
    r = (d - i)*60.0
    m = math.floor(r)
    s = (r - m)*60
    return int(i), int(m), int(s*100)

class Album(models.Model):

    the_parent = models.ForeignKey('Album', on_delete=models.SET_NULL, null=True)
    name = models.CharField(max_length=60)
    slug = models.CharField(max_length=60)
    first_timestamp = models.DateTimeField()
    last_timestamp = models.DateTimeField()
    time_offset = models.SmallIntegerField(null=True)
    the_photo = models.ForeignKey('Photo', on_delete=models.SET_NULL, null=True)

    def get_url_path(self):
        url = ''
        a = self
        while a is not None:
            url = a.slug + '/' + url
            a = a.the_parent
        return PREFIX + '/' + url

    def __str__(self):
        s = '%s (%s): %s-%s' % (self.name, self.slug,
                                self.first_timestamp.isoformat(),
                                self.last_timestamp.isoformat())
        a = self.the_parent
        while a is not None:
            s = '%s > %s' % (a.slug, s)
            a = a.the_parent
        return s

    class Meta:
        indexes = [ models.Index(fields=['the_parent',
                                         'slug',
                                         'first_timestamp',
                                         'last_timestamp',
                                         'the_photo']) ]
        ordering = ('first_timestamp',)

class Photo(models.Model):

    title = models.CharField(max_length=60)
    description = models.CharField(max_length=250)
    the_album = models.ForeignKey('Album', on_delete=models.CASCADE)
    location = models.CharField(max_length=250)
    latitude = models.FloatField(null=True)
    longitude = models.FloatField(null=True)
    width = models.SmallIntegerField(null=True)
    height = models.SmallIntegerField(null=True)
    export_width = models.SmallIntegerField(null=True)
    export_height = models.SmallIntegerField(null=True)
    view_width = models.SmallIntegerField(null=True)
    view_height = models.SmallIntegerField(null=True)
    thumb_width = models.SmallIntegerField(null=True)
    thumb_height = models.SmallIntegerField(null=True)
    timestamp = models.DateTimeField()
    view_timestamp = models.CharField(max_length=60)
    time_offset = models.SmallIntegerField(null=True)

    url = models.CharField(max_length=250, null=True)
    export_url = models.CharField(max_length=250, null=True)
    view_url = models.CharField(max_length=250, null=True)
    thumb_url = models.CharField(max_length=250, null=True)
    basename = models.CharField(max_length=100, null=True)
    view_latitude = models.CharField(max_length=50, null=True)
    view_longitude = models.CharField(max_length=50, null=True)

    def fill_aux_data(self):
        self.url = self.location.replace(STATIC_PATH_PREFIX, '')
        self.export_url = self.url.replace('/photos/', '/exports/')
        self.view_url = self.url.replace('/photos/', '/views/')
        self.thumb_url = self.url.replace('/photos/', '/thumbs/')
        self.basename = os.path.basename(self.location)
        locale =  LANGUAGE_CODE.replace('-' , '_')
        self.view_timestamp = format_datetime(self.timestamp, locale=locale,
                                              format='MMMM d, yyyy, HH:mm:ss') \
            .capitalize()
        lat = math.fabs(self.latitude)
        lon = math.fabs(self.longitude)
        lat_i, lat_m, lat_s = float2degrees(lat)
        lon_i, lon_m, lon_s = float2degrees(lon)
        lat_ref = 'S' if (self.latitude  < 0.0) else 'N';
        lon_ref = 'W' if (self.longitude < 0.0) else 'E';
        self.view_latitude = '%d° %d\' %2.2f" %s' % \
            (lat_i, lat_m, lat_s / 100.0, lat_ref)
        self.view_longitude = '%d° %d\' %2.2f" %s' % \
            (lon_i, lon_m, lon_s / 100.0, lon_ref)

    def localdatetime(self):
        tz = timezone(timedelta(hours=self.time_offset))
        return datetime(self.timestamp.year, self.timestamp.month,
                        self.timestamp.day, self.timestamp.hour,
                        self.timestamp.minute, self.timestamp.second,
                        self.timestamp.microsecond,
                        timezone(timedelta(hours=self.time_offset)))

    def get_url_path(self):
        return self.the_album.get_url_path() + self.basename

    def __str__(self):
        s = '%s (%s): %04d/%02d/%02d %02d:%02d:%02d' % \
            (self.title, self.basename,
             self.timestamp.year, self.timestamp.month,
             self.timestamp.day, self.timestamp.hour,
             self.timestamp.minute, self.timestamp.second)

        a = self.the_album
        while a is not None:
            s = '%s > %s' % (a.slug, s)
            a = a.the_parent
        return s

    class Meta:
        indexes = [ models.Index(fields=['timestamp',
                                         'location']) ]
        ordering = ('timestamp',)

class UploadedImage(models.Model):

    files = models.ImageField(upload_to=UPLOAD_TMP_DIR,
                              blank=True, null=True)
