import os
import os.path
import sys

from django.contrib.auth import get_user, authenticate, \
    logout as django_logout, login as django_login
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator
from django.db import transaction
from django.db.models import Q
from django.http import HttpResponse, Http404, \
    HttpResponseBadRequest,  JsonResponse
from django.shortcuts import render, redirect
from django.utils.http import urlencode
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.edit import FormView

from rest_framework.authentication import TokenAuthentication
from rest_framework.authtoken.models import Token
from rest_framework.decorators import api_view, authentication_classes, \
    parser_classes, permission_classes
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from galeria.settings import PREFIX, GALLERY_INFO, LANGUAGE_CODE, \
    STATIC_URL_PREFIX, PAGINATION_ITEMS

from fotos.models import Album, Photo, UploadedImage
from fotos.forms import UploadedImageForm
from fotos.util import get_image_metadata, normalize
from fotos.serializers import AlbumSerializer, PhotoSerializer, \
    UploadedImageSerializer

from datetime import datetime
from babel.dates import get_month_names

def get_month_name(ts, capitalize=False):
    locale = LANGUAGE_CODE.replace('-', '_')
    name = get_month_names(locale=locale)[ts.month]
    if capitalize:
        name = name.capitalize()
    return name

def _get_year_album(year):
    return Album.objects.filter(the_parent=None, slug=year).first()

def _get_month_album(ya, month):
    return Album.objects.filter(the_parent=ya, slug=month).first()

def _get_event_album(ma, event):
    return Album.objects.filter(the_parent=ma, slug=event).first()

def _get_photo(ea, photo):
    return Photo.objects.filter(the_album=ea, basename=photo).first()

def _get_years():
    return Album.objects.filter(the_parent=None).order_by('first_timestamp')

def _get_months(ya):
    return Album.objects.filter(the_parent=ya).order_by('first_timestamp')

def _get_events(ma):
    return Album.objects.filter(the_parent=ma).order_by('first_timestamp')

def _get_photos(ea):
    return Photo.objects.filter(the_album=ea).order_by('timestamp')

def _get_results(s, realm=''):
    if s == '':
        return Photo.objects.filter(id=-1)
    t = realm.split('/')
    if len(t) > 3:
        return None
    if realm == '':
        return Photo.objects.filter(Q(title__icontains=s)         |
                                    Q(description__icontains=s)   |
                                    Q(basename__icontains=s)      |
                                    Q(the_album__name__icontains=s)) \
                            .order_by('timestamp')
    if len(t) == 1:
        ya = _get_year_album(t[0])
        if ya is None:
            return None
        return Photo.objects.filter(Q(title__icontains=s)         |
                                    Q(description__icontains=s)   |
                                    Q(basename__icontains=s)      |
                                    Q(the_album__name__icontains=s),
                                    the_album__the_parent__the_parent=ya) \
                            .order_by('timestamp')
    if len(t) == 2:
        ya = _get_year_album(t[0])
        ma = _get_month_album(ya, t[1])
        if ya is None or ma is None:
            return None
        return Photo.objects.filter(Q(title__icontains=s)         |
                                    Q(description__icontains=s)   |
                                    Q(basename__icontains=s)      |
                                    Q(the_album__name__icontains=s),
                                    the_album__the_parent__the_parent=ya,
                                    the_album__the_parent=ma) \
                            .order_by('timestamp')
    if len(t) == 3:
        ya = _get_year_album(t[0])
        ma = _get_month_album(ya, t[1])
        ea = _get_month_album(ma, t[2])
        if ya is None or ma is None or ea is None:
            return None
        return Photo.objects.filter(Q(title__icontains=s)         |
                                    Q(description__icontains=s)   |
                                    Q(basename__icontains=s)      |
                                    Q(the_album__name__icontains=s),
                                    the_album__the_parent__the_parent=ya,
                                    the_album__the_parent=ma,
                                    the_album=ea) \
                            .order_by('timestamp')

def _get_albums_highlighted_by(phs):
    albums = Album.objects.filter(the_photo=phs[0])
    for ph in phs[1:]:
        a = Album.objects.filter(the_photo=ph)
        if a.count() > 0:
            albums = albums.union(a)
    return albums

def _get_prev_year_url(ya):
    prev_year = _get_prev_year(ya)
    if prev_year is not None:
        return prev_year.get_url_path()
    return None

def _get_next_year_url(ya):
    next_year = _get_next_year(ya)
    if next_year is not None:
        return next_year.get_url_path()
    return None

def _get_prev_month_url(ma):
    prev_month = _get_prev_month(ma)
    if prev_month is not None:
        return prev_month.get_url_path()
    return None

def _get_next_month_url(ma):
    next_month = _get_next_month(ma)
    if next_month is not None:
        return next_month.get_url_path()
    return None

def _get_prev_event_url(ea):
    prev_event = _get_prev_event(ea)
    if prev_event is not None:
        return prev_event.get_url_path()
    return None

def _get_next_event_url(ea):
    next_event = _get_next_event(ea)
    if next_event is not None:
        return next_event.get_url_path()
    return None

def _get_prev_photo_url(ph):
    prev_photo = _get_prev_photo(ph)
    if prev_photo is not None:
        return prev_photo.get_url_path()
    return None

def _get_next_photo_url(ph):
    next_photo = _get_next_photo(ph)
    if next_photo is not None:
        return next_photo.get_url_path()
    return None

def _get_prev_year(ya):
    years = _get_years()
    if years is None or years.first() == ya:
        return None
    i = 1
    while years[i] != ya:
        i += 1
    return years[i-1]

def _get_next_year(ya):
    years = _get_years()
    if years is None or years.last() == ya:
        return None
    i = years.count() - 2
    while years[i] != ya:
        i -= 1
    return years[i+1]

def _get_prev_month(ma):
    ya = ma.the_parent
    months = _get_months(ya)
    if ma == months.first():
        prev_year = _get_prev_year(ya)
        if prev_year is None:
            return None
        prev_year_months = _get_months(prev_year)
        return prev_year_months.last()
    i = 1
    while months[i] != ma:
        i += 1
    return months[i-1]

def _get_next_month(ma):
    ya = ma.the_parent
    months = _get_months(ya)
    if ma == months.last():
        next_year = _get_next_year(ya)
        if next_year is None:
            return None
        next_year_months = _get_months(next_year)
        return next_year_months.first()
    i = months.count() - 2
    while months[i] != ma:
        i -= 1
    return months[i+1]

def _get_prev_event(ea):
    ma = ea.the_parent
    events = _get_events(ma)
    if ea == events.first():
        prev_month = _get_prev_month(ma)
        if prev_month is None:
            return None
        prev_month_events = _get_events(prev_month)
        return prev_month_events.last()
    i = 1
    while events[i] != ea:
        i += 1
    return events[i-1]

def _get_next_event(ea):
    ma = ea.the_parent
    events = _get_events(ma)
    if ea == events.last():
        next_month = _get_next_month(ma)
        if next_month is None:
            return None
        next_month_events = _get_events(next_month)
        return next_month_events.first()
    i = events.count() - 2
    while events[i] != ea:
        i -= 1
    return events[i+1]

def _get_prev_photo(ph):
    ea = ph.the_album
    photos = _get_photos(ea)
    if ph == photos.first():
        prev_event = _get_prev_event(ea)
        if prev_event is None:
            return None
        prev_event_photos = _get_photos(prev_event)
        return prev_event_photos.last()
    i = 1
    while photos[i] != ph:
        i += 1
    return photos[i-1]

def _get_next_photo(ph):
    ea = ph.the_album
    photos = _get_photos(ea)
    if ph == photos.last():
        next_event = _get_next_event(ea)
        if next_event is None:
            return None
        next_event_photos = _get_photos(next_event)
        return next_event_photos.first()
    i = photos.count() - 2
    while photos[i] != ph:
        i -= 1
    return photos[i+1]

def _reset_album(album, old_highlight, new_highlight):
    album.refresh_from_db()
    if album.the_photo == old_highlight:
        album.the_photo = new_highlight
        album.save()

def _reset_event(year, month, event, ea=None):
    if ea is None:
        ya = _get_year_album(year)
        ma = _get_month_album(ya, month)
        ea = _get_event_album(ma, event)
    old_highlight = ea.the_photo
    new_highlight = _get_photos(ea).first()
    _reset_album(ea.the_parent.the_parent, old_highlight, new_highlight)
    _reset_album(ea.the_parent, old_highlight, new_highlight)
    _reset_album(ea, old_highlight, new_highlight)

def _reset_month(year, month, ma=None):
    if ma is None:
        ya = _get_year_album(year)
        ma = _get_month_album(ya, month)
    events = _get_events(ma)
    for ea in events:
        ea.the_photo = _get_photos(ea).first()
        ea.save()
    old_highlight = ma.the_photo
    new_highlight = events.first().the_photo
    _reset_album(ma.the_parent, old_highlight, new_highlight)
    _reset_album(ma, ma.the_photo, new_highlight)

def _reset_year(year, ya=None):
    if ya is None:
        ya = _get_year_album(year)
    months = _get_months(ya)
    for ma in months:
        events = _get_events(ma)
        for ea in events:
            ea.the_photo = _get_photos(ea).first()
            ea.save()
        ma.the_photo = events.first().the_photo
        ma.save()
    old_highlight = ya.the_photo
    new_highlight = months.first().the_photo
    _reset_album(ya, old_highlight, new_highlight)

def _reset():
    for ya in _get_years():
        _reset_year(ya.slug, ya)

def _highlight_month(request, year, month):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    if ya is None:
        return redirect(PREFIX + '/')
    if ma is None:
        return redirect(ya.get_url_path())

    if ya.the_photo != ma.the_photo:
        ya.the_photo = ma.the_photo
        ya.save()
    return redirect(ma.get_url_path())

def _highlight_event(request, year, month, event):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None:
        return redirect(PREFIX + '/')
    if ma is None:
        return redirect(ya.get_url_path())
    if ea is None:
        return redirect(ma.get_url_path())

    if ma.the_photo != ea.the_photo:
        old_ph = ma.the_photo
        if ya.the_photo == old_ph:
            ya.the_photo = ea.the_photo
            ya.save()
        ma.the_photo = ea.the_photo
        ma.save()
    return redirect(ea.get_url_path())

def _highlight_photo(request, year, month, event, photo):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None:
        return redirect(PREFIX + '/')
    if ma is None:
        return redirect(ya.get_url_path())
    if ea is None:
        return redirect(ma.get_url_path())
    if ph is None:
        return redirect(ea.get_url_path())

    if ea.the_photo != ph:
        old_ph = ea.the_photo
        if ya.the_photo == old_ph:
            ya.the_photo = ph
            ya.save()
        if ma.the_photo == old_ph:
            ma.the_photo = ph
            ma.save()
        ea.the_photo = ph
        ea.save()
    return redirect(ph.get_url_path())

def _delete(path):
    t = path.split('/')
    while len(t) < 4:
        t.append('')

    year, month, event, photo = t
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)

    if ya is not None and ma is not None and \
       ea is not None and ph is not None:
        try:
            ya, ma, ea = _delete_photo(ya, ma, ea, ph)
        except Exception as e:
            sys.stderr.write('Error deleting: %s\n' % (str(e)))
            return None
        if ea is not None:
            return ea.get_url_path()
        if ma is not None:
            return ma.get_url_path()
        if ya is not None:
            return ya.get_url_path()
        return PREFIX + '/'

    if ya is not None and ma is not None and ea is not None:
        try:
            ya, ma = _delete_event(ya, ma, ea)
        except Exception as e:
            sys.stderr.write('Error deleting: %s\n' % (str(e)))
            return None
        if ma is not None:
            return ma.get_url_path()
        if ya is not None:
            return ya.get_url_path()
        return PREFIX + '/'

    if ya is not None and ma is not None:
        try:
            ya = _delete_month(ya, ma)
        except Exception as e:
            sys.stderr.write('Error deleting: %s\n' % (str(e)))
            return None
        if ya is not None:
            return ya.get_url_path()
        return PREFIX + '/'

    if ya is not None:
        _delete_year(ya)

    return None

@transaction.atomic
def _delete_year(ya):
    months = _get_months(ya)
    events = Album.objects.filter(the_parent__the_parent=ya)
    photos = Photo.objects.filter(the_album__the_parent__the_parent=ya)

    locations = []
    for ph in photos:
        locations.append(ph.location)
        locations.append(ph.location.replace('/photos/', '/exports/', 1))
        locations.append(ph.location.replace('/photos/', '/views/', 1))
        locations.append(ph.location.replace('/photos/', '/thumbs/', 1))

    for highlighted in _get_albums_highlighted_by(photos):
        highlighted.the_photo = None
        highlighted.save()
    for ph in photos:
        ph.delete()
    for ea in events:
        ea.delete()
    for ma in months:
        ma.delete()

    ya.delete()

    for location in locations:
        os.remove(location)

@transaction.atomic
def _delete_month(ya, ma):
    events = _get_events(ma)
    photos = Photo.objects.filter(the_album__the_parent=ma)

    locations = []
    for ph in photos:
        locations.append(ph.location)
        locations.append(ph.location.replace('/photos/', '/exports/', 1))
        locations.append(ph.location.replace('/photos/', '/views/', 1))
        locations.append(ph.location.replace('/photos/', '/thumbs/', 1))

    for highlighted in _get_albums_highlighted_by(photos):
        highlighted.the_photo = None
        highlighted.save()
    for ph in photos:
        ph.delete()
    for ea in events:
        ea.delete()

    ma.delete()
    ma = None

    ya.refresh_from_db()
    months = _get_months(ya)
    if months.count() == 0:
        ya.delete()
        ya = None
    elif ya.the_photo is None:
        ya.the_photo = months.first().the_photo
        ya.save()

    for location in locations:
        os.remove(location)
    return ya

@transaction.atomic
def _delete_event(ya, ma, ea):
    locations = []

    photos = _get_photos(ea)
    for ph in photos:
        locations.append(ph.location)
        locations.append(ph.location.replace('/photos/', '/exports/', 1))
        locations.append(ph.location.replace('/photos/', '/views/', 1))
        locations.append(ph.location.replace('/photos/', '/thumbs/', 1))

    for highlighted in _get_albums_highlighted_by(photos):
        highlighted.the_photo = None
        highlighted.save()
    for ph in photos:
        ph.delete()

    ea.delete()
    ea = None

    ma.refresh_from_db()
    events = _get_events(ma)
    if events.count() == 0:
        ma.delete()
        ma = None
    elif ma.the_photo is None:
        ma.the_photo = events.first().the_photo
        ma.save()

    ya.refresh_from_db()
    months = _get_months(ya)
    if months.count() == 0:
        ya.delete()
        ya = None
    elif ya.the_photo is None:
        ya.the_photo = months.first().the_photo
        ya.save()

    for location in locations:
        os.remove(location)
    return ya, ma

@transaction.atomic
def _delete_photo(ya, ma, ea, ph):
    locations = [ ph.location,
                  ph.location.replace('/photos/', '/exports/', 1),
                  ph.location.replace('/photos/', '/views/', 1),
                  ph.location.replace('/photos/', '/thumbs/', 1) ]

    for highlighted in _get_albums_highlighted_by([ph]):
        highlighted.the_photo = None
        highlighted.save()
    ph.delete()
    ph = None

    ea.refresh_from_db()
    photos = _get_photos(ea)
    if photos.count() == 0:
        ea.delete()
        ea = None
    elif ea.the_photo is None:
        ea.the_photo = photos.first()
        ea.save()

    ma.refresh_from_db()
    events = _get_events(ma)
    if events.count() == 0:
        ma.delete()
        ma = None
    elif ma.the_photo is None:
        ma.the_photo = events.first().the_photo
        ma.save()

    ya.refresh_from_db()
    months = _get_months(ya)
    if months.count() == 0:
        ya.delete()
        ya = None
    elif ya.the_photo is None:
        ya.the_photo = months.first().the_photo
        ya.save()

    for location in locations:
        os.remove(location)
    return ya, ma, ea

def index(request):
    link = message = ''
    page_title = 'Galería'
    albums = _get_years()
    if albums.count() == 0:
        page_title = 'La galería está vacía'
        if request.user.is_authenticated:
            link = PREFIX + '/admin/upload'
            message = 'Comience a subir fotos si así lo desea'
        else:
            link = PREFIX + '/admin/login'
            message = 'Ingrese al sistema para poder subir fotos'
        summary = 'No se han agregado fotos'
    elif albums.count() == 1:
        summary = 'Un año'
    else:
        summary = '{:,} años'.format(albums.count())
    reset_url = PREFIX + '/admin/reset?' + urlencode({'path': '/'})
    attrs = { 'albums': albums,
              'breadcrumbs': [],
              'gallery': GALLERY_INFO,
              'link': link,
              'menus': [{ 'link': reset_url,
                          'icon': 'window-restore',
                          'name': 'Restablecer destacados en la galería' }],
              'message': message,
              'navigation': { 'prev': None, 'next': None, 'up': None },
              'page_title': page_title,
              'path': PREFIX,
              'prefix': PREFIX,
              'search': { 'realm': '' },
              'summary': summary,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def year(request, year):
    ya = _get_year_album(year)
    if ya is None:
        raise Http404(year + ' does not exist')
    months = _get_months(ya)
    path = ya.get_url_path().replace(PREFIX + '/', '', 1)
    reset_url = PREFIX + '/admin/reset?' + urlencode({'path': path})
    delete_url = PREFIX + '/admin/delete?' + urlencode({'path': path})

    if months.count() == 1:
        summary = 'Un mes'
    else:
        summary = '{:,} meses'.format(months.count())

    highlight = ya.the_photo.the_album.the_parent
    attrs = { 'album': ya,
              'albums': months,
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name } ],
              'gallery': GALLERY_INFO,
              'highlight': highlight,
              'menus': [{ 'link': reset_url,
                          'icon': 'window-restore',
                          'name': 'Restablecer destacados en año' },
                        { 'link': delete_url,
                          'icon': 'trash',
                          'name': 'Eliminar año' }],
              'navigation': { 'prev': _get_prev_year_url(ya),
                              'next': _get_next_year_url(ya),
                              'up': PREFIX + '/#' + year },
              'page_title': 'Año ' + year,
              'path': PREFIX + '/' + year,
              'prefix': PREFIX,
              'search': { 'realm': year },
              'summary': summary,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def month(request, year, month):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    if ya is None or ma is None:
        raise Http404(os.path.join(year, month) + ' does not exist')

    events = _get_events(ma)
    n = events.count()
    if n == 1:
        summary = 'Un evento'
    else:
        summary = '{:,} eventos'.format(n)

    page_title = ('Mes de %s, %d' % (get_month_name(ma.first_timestamp),
                                     ya.first_timestamp.year))
    highlighted = (ya.the_photo == ma.the_photo)
    path = ma.get_url_path().replace(PREFIX + '/', '', 1)
    reset_url = PREFIX + '/admin/reset?' + urlencode({'path': path})
    highlight_url = PREFIX + '/admin/highlight?' + urlencode({'path': path})
    delete_url = PREFIX + '/admin/delete?' + urlencode({'path': path})
    highlight = ma.the_photo.the_album

    page = request.GET.get('page', 1)
    paginator = Paginator(events, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    attrs = { 'album': ma,
              'albums': the_page,
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name } ],
              'gallery': GALLERY_INFO,
              'highlight': highlight,
              'highlighted': highlighted,
              'menus': [{ 'link': reset_url,
                          'icon': 'window-restore',
                          'name': 'Restablecer destacados en mes' },
                        { 'link': highlight_url,
                          'icon': 'check-circle',
                          'name': 'Destacar mes'},
                        { 'link': delete_url,
                          'icon': 'trash',
                          'name': 'Eliminar mes' }],
              'navigation': { 'prev': _get_prev_month_url(ma),
                              'next': _get_next_month_url(ma),
                              'up':  ya.get_url_path() + '#' + month },
              'page': the_page,
              'page_title': page_title,
              'path': os.path.join(PREFIX + '/', year, month),
              'prefix': PREFIX,
              'search': { 'realm': os.path.join(year, month) },
              'summary': summary,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def event(request, year, month, event):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None or ma is None or ea is None:
        raise Http404(os.path.join(year, month, event) + ' does not exist')

    photos = _get_photos(ea)
    n = photos.count()
    if n == 1:
        summary = 'Una foto'
    else:
        summary = '{:,} fotos'.format(n)
    highlighted = (ma.the_photo == ea.the_photo)
    path = ea.get_url_path().replace(PREFIX + '/', '', 1)
    map_url = PREFIX + '/admin/map?' + urlencode({'path': path})
    reset_url = PREFIX + '/admin/reset?' + urlencode({'path': path})
    highlight_url = PREFIX + '/admin/highlight?' + urlencode({'path': path})
    delete_url = PREFIX + '/admin/delete?' + urlencode({'path': path})
    highlight = ea.the_photo

    up = ma.get_url_path()
    events = _get_events(ma)
    i = 0
    while ea != events[i]:
        i += 1
    up_page = 1 + int(float(i) / PAGINATION_ITEMS)
    if up_page > 1:
        up += '?page=%d' % up_page

    page = request.GET.get('page', 1)
    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    attrs = { 'album': ea,
              'breadcrumbs': [{ 'link': ya.get_url_path(),
                                'name': ya.name },
                              { 'link': ma.get_url_path(),
                                'name': ma.name },
                              { 'link': ea.get_url_path(),
                                'name': ea.name }],
              'gallery': GALLERY_INFO,
              'highlight': highlight,
              'highlighted': highlighted,
              'menus': [{ 'link': map_url,
                          'icon': 'map',
                          'name': 'Mapa del evento' },
                        { 'link': reset_url,
                          'icon': 'window-restore',
                          'name': 'Restablecer destacados en evento' },
                        { 'link': highlight_url,
                          'icon': 'check-circle',
                          'name': 'Destacar evento' },
                        { 'link': delete_url,
                          'icon': 'trash',
                          'name': 'Eliminar evento' }],
              'navigation': { 'prev': _get_prev_event_url(ea),
                              'next': _get_next_event_url(ea),
                              'up': up + '#' + ea.slug },
              'page': the_page,
              'page_title': ea.name,
              'photos': the_page,
              'path': os.path.join(PREFIX + '/', year, month, event),
              'prefix': PREFIX,
              'search': { 'realm': os.path.join(year, month, event) },
              'summary': summary,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def photo(request, year, month, event, photo):
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None or ma is None or ea is None or ph is None:
        raise Http404(os.path.join(year, month, event, photo +
                                   ' does not exist'))
    bgcolor = 'bgnormal' if ea.the_photo != ph else 'bghighlight'
    path = ph.get_url_path().replace(PREFIX + '/', '', 1)
    highlight_url = PREFIX + '/admin/highlight?' + urlencode({'path': path})
    delete_url = PREFIX + '/admin/delete?' + urlencode({'path': path})

    up = ea.get_url_path()
    photos = _get_photos(ea)
    i = 0
    while ph != photos[i]:
        i += 1
    up_page = 1 + int(float(i) / PAGINATION_ITEMS)
    if up_page > 1:
        up += '?page=%d' % up_page

    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': ph.get_url_path(),
                                 'name': ph.title } ],
              'bgcolor': bgcolor,
              'gallery': GALLERY_INFO,
              'menus': [{ 'link': highlight_url,
                          'icon': 'check-circle',
                          'name': 'Destacar foto' },
                        { 'link': delete_url,
                          'icon': 'trash',
                          'name': 'Eliminar foto' }],
              'navigation': { 'prev': _get_prev_photo_url(ph),
                              'next': _get_next_photo_url(ph),
                              'up':  up + '#' + ph.basename },
              'page_title': ph.title,
              'photo': ph,
              'path': os.path.join(PREFIX, year, month, event),
              'prefix': PREFIX,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

def search(request):
    s = request.GET.get('s', '')
    realm = request.GET.get('realm', '')
    page = request.GET.get('page', 1)
    link = message = ''
    photos = _get_results(s, realm)
    if photos is None:
        raise Http404('Invalid realm')
    n = photos.count()
    if n == 0:
        link = PREFIX + '/'
        message = 'No hay fotos que cacen la búsqueda'
        page_title = 'Búsqueda sin resultados'
    else:
        page_title = 'Resultados de la búsqueda: “%s”' % s
        if n == 1:
            summary = 'Una foto'
        else:
            summary = '{:,} fotos'.format(n)

    qs = request.META.get('QUERY_STRING', '')

    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    bc_link = PREFIX + '/search/realm=%s&s=%s&page=%s' % (realm, s, page)
    attrs = { 'breadcrumbs': [{ 'link': request.get_full_path(),
                                'name': 'Búsqueda: “%s”' % s }],
              'gallery': GALLERY_INFO,
              'link': link,
              'message': message,
              'navigation': { 'prev': None, 'next': None,
                              'up': PREFIX + '/' + realm },
              'page': the_page,
              'page_title': page_title,
              'path': PREFIX + '/result/?' + qs + '&path=',
              'results': the_page,
              'prefix': PREFIX,
              'summary': summary,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def result(request):
    path = request.GET.get('path', None)
    if path is None:
        raise Http404('Missing path')
    t = path.split('/')
    if len(t) < 4:
        raise Http404('Ivalid path ' + path)

    realm = request.GET.get('realm', '')
    s = request.GET.get('s', '')
    page = request.GET.get('page', '1')
    photos = _get_results(s, realm)
    n = photos.count()
    if n == 0:
        raise Http404('Ivalid result ' + request.get_full_path())
    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    year, month, event, photo = t[-4:]
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    ph = _get_photo(ea, photo)
    if ya is None or ma is None or ea is None or ph is None:
        raise Http404(os.path.join(year, month, event, photo +
                                   ' does not exist'))

    i = 0
    while ph != the_page[i]:
        i += 1

    p = PREFIX
    f = '%s/result/?realm=%s&s=%s&page=%s&path=%s%s'
    if i == 0:
        if the_page.has_previous():
            pp = str(int(page)-1)
            prev_page = paginator.get_page(the_page.previous_page_number())
            prev = f % (p, realm, s, pp, p, prev_page[-1].get_url_path())
        else:
            prev = None
        next = f % (p, realm, s, page, p, the_page[i+1].get_url_path())
    elif i == len(the_page) - 1:
        prev = f % (p, realm, s, page, p, the_page[i-1].get_url_path())
        if the_page.has_next():
            np = str(int(page)+1)
            next_page = paginator.get_page(the_page.next_page_number())
            next = f % (p, realm, s, np, p, next_page[0].get_url_path())
        else:
            next = None
    else:
        prev = f % (p, realm, s, page, p, the_page[i-1].get_url_path())
        next = f % (p, realm, s, page, p, the_page[i+1].get_url_path())

    up = PREFIX + '/search/?realm=%s&s=%s' % (realm, s)
    if page != '':
        up += '&page=%s' % page

    p = ph.get_url_path().replace(PREFIX + '/', '', 1)
    delete_url = PREFIX + '/admin/delete?' + urlencode({'path': p})


    search_link = PREFIX + '/search/realm=%s&s=%s&page=%s' % (realm, s, page)

    attrs = { 'bgcolor': 'bgnormal',
              'breadcrumbs': [ { 'link': search_link,
                                 'name': 'Búsquedas: “%s”' % s },
                               { 'link': request.get_full_path(),
                                 'name': ph.title }],
              'gallery': GALLERY_INFO,
              'menus': [ {'link': delete_url,
                          'icon': 'trash',
                          'name': 'Eliminar foto'}],
              'navigation': { 'prev': prev,
                              'next': next,
                              'up':  up + '#' + ph.basename },
              'page_title': ph.title,
              'photo': ph,
              'path': os.path.join(PREFIX, year, month, event),
              'prefix': PREFIX,
              'sup': STATIC_URL_PREFIX,
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

@transaction.atomic
def _get_photo_album(metadata):
    ts = metadata['timestamp']
    sy = '%04d' % ts.year
    sm = '%02d' % ts.month
    year_album = Album.objects.filter(the_parent=None, slug=sy).first()
    if year_album is None:
        year_album = Album(the_parent = None,
                           name = 'Año ' + sy,
                           slug = sy,
                           first_timestamp = ts,
                           last_timestamp = ts,
                           time_offset = metadata['time_offset'],
                           the_photo = None)
        year_album.save()
    month_album = Album.objects.filter(the_parent=year_album, slug=sm).first()
    if month_album is None:
        month_album = Album(the_parent = year_album,
                            name = get_month_name(ts, capitalize=True),
                            slug = sm,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        month_album.save()
    pa_slug = normalize(metadata['album'])
    event_album = Album.objects.filter(the_parent=month_album,
                                       slug=pa_slug).first()
    if event_album is None:
        event_album = Album(the_parent = month_album,
                            name = metadata['album'],
                            slug = pa_slug,
                            first_timestamp = ts,
                            last_timestamp = ts,
                            time_offset = metadata['time_offset'],
                            the_photo = None)
        event_album.save()
    return year_album, month_album, event_album

@transaction.atomic
def _add_photo(uploaded_image):
    path = str(uploaded_image.files.file)
    uploaded_image.delete()
    md = get_image_metadata(path)
    if md is None:
        os.remove(path)
        return
    year_album, month_album, event_album = _get_photo_album(md)
    photo = Photo(title = md['title'],
                  description = md['description'],
                  the_album = event_album,
                  location = md['location'],
                  latitude = md['latitude'],
                  longitude = md['longitude'],
                  width = md['width'],
                  height = md['height'],
                  export_width = md['export_width'],
                  export_height = md['export_height'],
                  view_width = md['view_width'],
                  view_height = md['view_height'],
                  thumb_width = md['thumb_width'],
                  thumb_height = md['thumb_height'],
                  timestamp = md['timestamp'],
                  time_offset = md['time_offset'])
    photo.fill_aux_data()
    photo.save()
    if event_album.first_timestamp > photo.timestamp:
        event_album.first_timestamp = photo.timestamp
    if event_album.last_timestamp < photo.timestamp:
        event_album.last_timestamp = photo.timestamp
    if event_album.the_photo is None:
        event_album.the_photo = photo
    event_album.save()
    if month_album.first_timestamp > event_album.first_timestamp:
        month_album.first_timestamp = event_album.first_timestamp
    if month_album.last_timestamp < event_album.last_timestamp:
        month_album.last_timestamp = event_album.last_timestamp
    if month_album.the_photo is None:
        month_album.the_photo = photo
    month_album.save()
    if year_album.first_timestamp > month_album.first_timestamp:
        year_album.first_timestamp = month_album.first_timestamp
    if year_album.last_timestamp < month_album.last_timestamp:
        year_album.last_timestamp = month_album.last_timestamp
    if year_album.the_photo is None:
        year_album.the_photo = photo
    year_album.save()
    os.remove(path)

@login_required
def info(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    return render(request, 'info.html',
                  { 'title': 'Información',
                    'gallery': GALLERY_INFO,
                    'prefix': PREFIX,
                    'token': token.key })

@login_required
def password(request):
    if request.method == 'POST':
        message = type = ''
        password = request.POST.get('password', '')
        if len(password) > 0:
            request.user.set_password(password)
            request.user.save()
            type = 'success'
            message = 'Contraseña actualizada.'
        else:
            type = 'failure'
            message = 'Contraseña inválida.'
        return render(request, 'password.html',
                      { 'title': 'Cambiar contraseña',
                        'gallery': GALLERY_INFO,
                        'type': type,
                        'message': message,
                        'prefix': PREFIX })
    else:
        return render(request, 'password.html',
                      { 'title': 'Cambiar contraseña',
                        'gallery': GALLERY_INFO,
                        'prefix': PREFIX })

@login_required
def upload(request):
    if request.method == 'POST':
        form = UploadedImageForm(request.POST, request.FILES)
        files = request.FILES.getlist('files')
        if form.is_valid():
            c = 0
            for f in files:
                c += 1
                uploaded_image = UploadedImage(files=f)
                uploaded_image.save()
                _add_photo(uploaded_image)
            if len(files) > 0:
                if c > 1:
                    message = (('Se subieron los %d ' % c) +
                               'archivos de forma exitosa.')
                else:
                    message = 'Se subió el archivo de forma exitosa.'
                return JsonResponse({ 'status': 'success',
                                      'message': message })
            return JsonResponse({ 'status': 'failure',
                                  'message': 'Solicitud inválida.' })
        return JsonResponse({ 'status': 'failure',
                              'message': 'Something went wrong.' })
    else:
        form = UploadedImageForm()
        return render(request, 'upload.html',
                      { 'form': form,
                        'gallery': GALLERY_INFO,
                        'prefix': PREFIX,
                        'title': 'Subir imágenes' })

@login_required
def map(request):
    path = request.GET.get('path', None)
    if path is None:
        raise Http404('Missing path')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = None
    t = path.split('/')
    if len(t) != 3:
        return redirect(request.META.get('HTTP_REFERER', PREFIX + '/'))
    year, month, event = t
    ya = _get_year_album(year)
    ma = _get_month_album(ya, month)
    ea = _get_event_album(ma, event)
    if ya is None or ma is None or ea is None:
        raise Http404('The requested map does not exist')
    photos = _get_photos(ea)
    markers = []
    for photo in photos:
        markers.append({ 'lab': photo.title,
                         'lat': photo.latitude,
                         'lng': photo.longitude })
    prev_url = PREFIX + '/admin/map?path=' + \
        _get_prev_event_url(ea).replace(PREFIX + '/', '')
    next_url = PREFIX + '/admin/map?path=' + \
        _get_next_event_url(ea).replace(PREFIX + '/', '')
    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': request.get_full_path(),
                                 'name': 'Mapa' } ],
              'prefix': PREFIX,
              'page_title': 'Mapa de ' + ea.name,
              'gallery': GALLERY_INFO,
              'markers': markers,
              'summary': '%d fotos' % photos.count(),
              'navigation': { 'prev': prev_url,
                              'next': next_url,
                              'up': ea.get_url_path() } }
    return render(request, 'map.html', attrs)

def logout(request):
    django_logout(request)
    return redirect(PREFIX + '/')

def login(request):
    if request.method == 'POST':
        username = request.POST.get('username', '')
        password = request.POST.get('password', '')
        user = authenticate(request, username=username, password=password)
        next = request.POST.get('next', None)
        if user is not None:
            django_login(request, user)
            if next is not None:
                return redirect(next)
            token = Token.objects.get_or_create(user=request.user)[0]
            return render(request, 'info.html',
                          { 'title': 'Información',
                            'gallery': GALLERY_INFO,
                            'message': 'Usuario %s autenticado.' % username,
                            'prefix': PREFIX,
                            'token': token.key,
                            'type': 'success' })
        return render(request, 'login.html',
                      { 'title': 'Entrar a la Galería',
                        'gallery': GALLERY_INFO,
                        'message': 'Contraseña o usuario incorrecto.',
                        'next': next,
                        'prefix': PREFIX,
                        'type': 'failure' })
    else:
        next = request.GET.get('next', '')
        if next == '':
            next = PREFIX
        return render(request, 'login.html',
                      { 'title': 'Entrar a la Galería',
                        'gallery': GALLERY_INFO,
                        'prefix': PREFIX,
                        'next': next })

@login_required
def reset(request):
    path = request.GET.get('path', '')
    if path == '/':
        _reset()
        return redirect(PREFIX + '/' + path)
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = None
    t = path.split('/')
    if len(t) == 3:
        year, month, event = t
        _reset_event(year, month, event)
    elif len(t) == 2:
        year, month = t
        _reset_month(year, month)
    elif len(t) == 1:
        year = t[0]
        _reset_year(year)
    return redirect(PREFIX + '/' + path)

@login_required
def highlight(request):
    path = request.GET.get('path', '')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = photo = None
    t = path.split('/')
    if len(t) == 4:
        year, month, event, photo = t
        return _highlight_photo(request, year, month, event, photo)
    elif len(t) == 3:
        year, month, event = t
        return _highlight_event(request, year, month, event)
    elif len(t) == 2:
        year, month = t
        return _highlight_month(request, year, month)
    return redirect(PREFIX + '/')

@login_required
def delete(request):
    path = request.GET.get('path', '')
    redir = _delete(path)
    if redir is not None:
        return redirect(redir)
    return redirect(PREFIX + '/')

@api_view(['GET'])
def list_albums(request):
    albums = Album.objects.all()
    serializer = AlbumSerializer(albums, many=True)
    return Response(serializer.data)

@api_view(['GET'])
def list_photos(request):
    photos = Photo.objects.all()
    serializer = PhotoSerializer(photos, many=True)
    return Response(serializer.data)

@api_view(['GET'])
def random_photo(request):
    photo = Photo.objects.all().order_by('?').first()
    klass = 'gthumb' if photo.thumb_width > photo.thumb_height else 'gthumbv'
    image = STATIC_URL_PREFIX + photo.thumb_url
    link = photo.get_url_path()
    return JsonResponse({ 'class': klass,
                          'image': image,
                          'link': link,
                          'title': photo.title })

@api_view(['POST'])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def upload_photo(request):
    if request.method != 'POST':
        return JsonResponse({'response': 'Invalid request'})
    form = UploadedImageSerializer(request.POST, request.FILES)
    files = request.FILES.getlist('files')
    if form.is_valid():
        for f in files:
            uploaded_image = UploadedImage(files=f)
            uploaded_image.save()
            _add_photo(uploaded_image)
        return JsonResponse({'response': 'OK'})
    else:
        return JsonResponse({'response': 'Error trying to upload photo'})

@api_view(['GET'])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def delete_item(request):
    path = request.GET.get('path', '')
    if _delete(path) is not None:
        return JsonResponse({'response': 'OK'})
    return JsonResponse({'response': 'Error: Invalid path %s' % path})

@api_view(['GET'])
@parser_classes([JSONParser])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def reset_highlights(request):
    for ya in _get_years():
        _reset_year(ya.slug, ya)
    return JsonResponse({'response': 'OK'})

@api_view(['GET'])
def get_highlights(request):
    r = {}
    years = _get_years()
    for year in years:
        r[year.slug] = { 'months': {} }
        months = _get_months(year)
        for month in months:
            ms = r[year.slug]['months']
            ms[month.slug] = { 'events': {} }
            events = _get_events(month)
            for event in events:
                es = ms[month.slug]['events']
                es[event.slug] = { 'highlight': event.the_photo.basename }
                es[event.slug]['is_highlight'] = \
                    month.the_photo == event.the_photo
            ms[month.slug]['is_highlight'] = \
                year.the_photo == month.the_photo
    return JsonResponse(r)

@api_view(['POST'])
@parser_classes([JSONParser])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def set_highlights(request):
    if request.method != 'POST':
        return JsonResponse({'response': 'Invalid request'})
    highlights = request.data
    for year in highlights.keys():
        ya = _get_year_album(year)
        if ya is None:
            return JsonResponse({'response': 'Invalid year: %s' % year})
        months = highlights[year]['months']
        for month in months.keys():
            ma = _get_month_album(ya, month)
            if ma is None:
                return JsonResponse({'response': 'Invalid month: %s' % month})
            events = months[month]['events']
            for event in events.keys():
                ea = _get_event_album(ma, event)
                if ea is None:
                    return JsonResponse({'response':
                                         'Invalid event: %s' % event})
                photo = events[event]['highlight']
                ph = _get_photo(ea, photo)
                if ph is None:
                    return JsonResponse({'response':
                                         'Invalid photo: %s' % photo})
                ea.the_photo = ph
                ea.save()
            for event in events.keys():
                ea = _get_event_album(ma, event)
                if events[event]['is_highlight']:
                    ma.the_photo = ea.the_photo
                    ma.save()
        for month in months.keys():
            ma = _get_month_album(ya, month)
            if months[month]['is_highlight']:
                ya.the_photo = ma.the_photo
                ya.save()
    return JsonResponse({'response': 'OK'})
